/*                                    palt
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>
#include <time.h>
#include <errno.h>
#include <math.h>

#undef DOS                             /* gcc compiler */
/* #define DOS                            /* DOS compiler */

#ifdef DOS

   #include <conio.h>
   #include <process.h>
   #include <dos.h>
   #include <bios.h>

   #define R_OK   04
   #define access _access

#else
   #include <unistd.h>
#endif

#include "getopt.h"

#define     TRUE           1
#define     FALSE          0

   typedef  unsigned char  byte ;
   typedef  unsigned short u16 ;
   typedef  unsigned long  u32 ;

#define     FEET           0
#define     METERS         1

double  H_Convert [] =
        {
              1.0,                  /* FEET     */
              3.28084,              /* METERS   */
        } ;

char H_Units [] = "fm" ;

#define     INHG           0
#define     TORR           1
#define     PSI            2
#define     PSF            3
#define     PA             4
#define     KPA            5
#define     MILLIBAR       6
#define     A5100          7
#define     A4100          8
#define     RATIO          9

double  P_Convert [] =
        {
              1.0,                  /* INHG     */
             25.4,                  /* TORR     */
              0.491154,             /* PSI      */
             70.726176,             /* PSF      */
           3386.379,                /* PA       */
              3.386379,             /* KPA      */
             33.86379,              /* MILLIBAR */
              1.0000,               /* A5100    */
              1.0000,               /* A4100    */
              1.0000,               /* RATIO    */
        } ;

char P_Units [] = "htifpkmanr" ;

#define P_ZERO         29.92134

   extern   char   * optarg ;             /* required for lib fun getarg */
   extern   int      optind ;             /* ditto                       */

   char   * progname       ;              /* who am i, anyway ?          */
   char   * proghome       ;              /* and where am i ?            */

#ifdef DOS
   #define   DIR_SEP             '\\'     /* change for unix / mac / dos */
#else
   #define   DIR_SEP             '/'      /* change for unix / mac / dos */
#endif

/* fine   PALT_GAIN_5100         0.114129       /* this is the 5100 xducer */
/* fine   PALT_OFFSET_5100       5.8473
/* fine   PALT_GAIN_4100         0.1123728947   /* this is the 4100 xducer */
/* fine   PALT_OFFSET_4100       3.4710         /* these are average lines */

#define   PALT_GAIN_5100         0.1447552322   /* this is the 5100 xducer */
#define   PALT_OFFSET_5100      -0.478599
#define   PALT_GAIN_4100         0.1113501786   /* this is the 4100 xducer */
#define   PALT_OFFSET_4100       3.418657       /* these are average lines */

#define   PRESSURE               0              /* default:   report p @ a */
#define   ALTITUDE               1              /* option i:  report a @ p */

#define   MAXALT                 36093
#define   MINRAT               0.22299

char  *  GetMyName ( char *, char * ) ;
char  *  ToLower ( char * ) ;
void     Usage ( void ) ;
double   PConvert ( double, int ) ;
u16      GetUnit ( char *, char * ) ;

/* ------------------------------------------------------------------------ */
main ( argc, argv )
/* ------------------------------------------------------------------------ */
  int      argc;
  char   * argv [];
/* ------------------------------------------------------------------------ */
{
   int    Mode       = INHG ;
   int    Compute    = PRESSURE ;

   double BasePress  = P_ZERO ;
   int    BaseUnits  = INHG ;

   double T          = 0 ;

   double P0         = P_ZERO ;

   double Alt        = 0 ;
   int    PressUnits = INHG ;

   char   * EoB      = NULL ;

   int    o ;

   int    Verbose = FALSE ;

   progname = GetMyName ( progname, argv [0] ) ;

   while ( 1 )
   {
      o = getopt ( argc, argv, "acn45itmfpkrhvz:" ) ;

      if ( o == EOF )
        break ;

      switch (o)
      {
         case 'a':
         case '5':
            Mode = A5100 ;
            break ;

         case 'n':
         case '4':
            Mode = A4100 ;
            break ;

         case 'h':
            Mode = INHG ;
            break ;

         case 'p':
            Mode = PA ;
            break ;

         case 'k':
            Mode = KPA ;
            break ;

         case 't':
            Mode = TORR ;
            break ;

         case 'i':
            Mode = PSI ;
            break ;

         case 'f':
            Mode = PSF ;
            break ;

         case 'm':
            Mode = MILLIBAR ;
            break ;

         case 'r':
            Mode = RATIO ;
            break ;

         case 'z':
            BasePress =  strtod ( optarg, & EoB ) ;
            BaseUnits = GetUnit ( EoB, P_Units )  ;
            BasePress = PConvert ( BasePress, BaseUnits ) ;
            break ;

         case 'c':
            Compute = ALTITUDE ;
            break ;

         case 'v':
            Verbose = TRUE ;
            break ;

         case '?':
            Usage () ;
            break ;

      }
   }

   if ( Compute == PRESSURE )
   {
      if ( optind < argc )
      {
         Alt =  strtod ( argv [ optind ], & EoB ) ;
         Alt *= H_Convert [ GetUnit ( EoB, H_Units ) ] ;
      }

/*    if (( BasePress != P_ZERO ) &&
          ( BaseUnits == INHG ) &&
          ( Mode != INHG ))
                    BasePress = PConvert ( BasePress, Mode ) ;
 */
      if ( Mode == A4100 )
      {
         T = 1 - ( 0.00000688 * ( Alt )) ;

         P0 = BasePress * exp ( 5.256 * log ( T )) ;

         printf ( "%.4f\n", (( P0 - PALT_OFFSET_4100 ) / PALT_GAIN_4100 )) ;
      }
      else if ( Mode == A5100 )
      {
         T = 1 - ( 0.00000688 * ( Alt )) ;

         P0 = BasePress * exp ( 5.256 * log ( T )) ;

         printf ( "%.4f\n", (( P0 - PALT_OFFSET_5100 ) / PALT_GAIN_5100 )) ;
      }
      else if ( Mode == RATIO )
      {
         T = 1 - ( 0.00000688 * ( Alt )) ;

         P0 = exp ( 5.256 * log ( T )) ;

         printf ( "%.4f\n", P0 ) ;
      }
      else
      {
         P0 = BasePress * P_Convert [ Mode ] ;

         T = 1 - ( 0.00000688 * ( Alt )) ;

         printf ( "%.4f\n", P0 * exp ( 5.256 * log ( T ))) ;
      }
   }
   else
   {
      if ( optind < argc )
      {
         P0 =  strtod ( argv [ optind ], & EoB ) ;

         PressUnits = GetUnit ( EoB, P_Units ) ;

         P0 = PConvert ( P0, PressUnits ) ;

         if (( PressUnits == INHG ) && ( Mode != INHG ))
            P0 = PConvert ( P0, Mode ) ;

/*       if (( BasePress != P_ZERO ) &&
             ( BaseUnits == INHG ) &&
             ( Mode != INHG ))
                       BasePress = PConvert ( BasePress, Mode ) ;
 */
         if ( BasePress <= 0 )
            printf ( "0\n" ) ;
         else
         {
            if ( Verbose )
            {
               printf ( "Press      = %.4f inHg\n", P0 );
               printf ( "BasePress  = %.4f inHg\n", BasePress );
            }

            if ( PressUnits != RATIO )
               P0 /= BasePress ;

            if ( Verbose )
               printf ( "PressRatio = %.4f\n", P0 );

            if ( P0 < MINRAT )
            {
               if ( Verbose )
                  printf ( "Altitude   > %.0f ft\n", ( double ) MAXALT );
               else
                  printf ( "> %.0f\n", ( double ) MAXALT ) ;
            }
            else
            {
               Alt = ( 1 - ( exp ( log ( P0 ) / 5.256 ))) / 0.00000688 ;

               if ( Verbose )
                  printf ( "Altitude   = %.0f ft\n", Alt ) ;
               else
                  printf ( "%.0f\n", Alt );
            }
         }
      }
      else
      {
         printf ( "0\n" );
      }
   }

   exit ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
void  Usage ()
/* ------------------------------------------------------------------------ */
{
   fprintf ( stderr, "%s -- Returns Pressure at Altitude in several units\n\n",
             progname ) ;
   fprintf ( stderr, "Usage:  %s [ -t -i -f -m -k -p -z ] [ -z Press ] Alt\n",
                              progname );
   fprintf ( stderr, "where:\n" );
   fprintf ( stderr, "   -t       -> Pressure At Altitude in Torr\n" ) ;
   fprintf ( stderr, "   -i       -> Pressure At Altitude in PSI\n" ) ;
   fprintf ( stderr, "   -f       -> Pressure At Altitude in PSF\n" ) ;
   fprintf ( stderr, "   -m       -> Pressure At Altitude in Millibar\n" ) ;
   fprintf ( stderr, "   -k       -> Pressure At Altitude in KiloPascal\n" ) ;
   fprintf ( stderr, "   -p       -> Pressure At Altitude in Pascal\n" ) ;
   fprintf ( stderr, "   -r       -> Pressure Ratio for Alt Feet AGL\n" ) ;
   fprintf ( stderr, "   -c       -> Altitude at the given Pressure\n" ) ;
   fprintf ( stderr, "   -h       -> This List\n" ) ;
   fprintf ( stderr, "   -z Press -> Set Base Pressure to Press Units\n" ) ;
   fprintf ( stderr, "   Alt      -> Compute the pressure at altitude Alt\n" ) ;

   exit ( 6 ) ;

   /* note that the AltAcc Output is undoc'd */

}
/* ------------------------------------------------------------------------ */
char  * GetMyName ( Name,  ArgV )
/* ------------------------------------------------------------------------ */
      char * Name ;
      char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;

   /* Throw out the path */

   if (( Name = strrchr ( ArgV, DIR_SEP )) != NULL )
   {
      * Name = '\0' ;                  /* delimit the path from the name */
        Name++;                        /* jump over it                   */
   }
   else
      Name = ArgV ;

#ifdef DOS

   /* throw away the '.exe' */

   if (( p = strrchr ( Name, '.' )) != NULL )
      * p = '\0' ;

   Name = ToLower ( Name ) ;

#endif

   return ( Name ) ;

}
/* ------------------------------------------------------------------------ */
char  * ToLower ( InStr )
/* ------------------------------------------------------------------------ */
      char * InStr ;
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;

   while ( i < strlen ( InStr ))
   {
      if ( isupper ( InStr [ i ] ))
         InStr [ i ]  = tolower ( InStr [ i ] ) ;

      i ++ ;
   }

   return ( InStr ) ;

}
/* ------------------------------------------------------------------------ */
double PConvert ( Press, Mode )
/* ------------------------------------------------------------------------ */
      double  Press ;
      int     Mode  ;
/* ------------------------------------------------------------------------ */
{
   if ( Mode == A5100 )
      Press = PALT_GAIN_5100 * Press + PALT_OFFSET_5100 ;
   else if ( Mode == A4100 )
      Press = PALT_GAIN_4100 * Press + PALT_OFFSET_4100 ;
   else
      Press /= P_Convert [ Mode ] ;

   return ( Press ) ;
}
/* ------------------------------------------------------------------------ */
u16   GetUnit ( InStr, UnitStr )
/* ------------------------------------------------------------------------ */
      char *  InStr ;
      char *  UnitStr ;
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;

   char * UnitPtr ;

   char * StrSep = " " ;

   UnitPtr = strtok ( InStr, StrSep ) ;

   if ( UnitPtr != NULL )
   {
      for ( i = 0 ; i < strlen ( UnitStr ) ; i ++ )
         if ( * UnitPtr == UnitStr [i] )
            return ( i ) ;
   }
   return ( 0 );
}
