/*
   This program is to calibrate the AltAcc and print results to a file
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>
#include <time.h>
#include <errno.h>
#include <math.h>

#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include "getopt.h"

#define       TRUE           1
#define       FALSE          0

   typedef  unsigned char        byte ;
   typedef  unsigned short       u16 ;

#ifdef  DOS
#define DPORT                "COM1"
#define BAUD                 B9600
#else
#define DPORT                "/dev/ttyS0"
#define BAUD                 B9600
#define _POSIX_SOURCE        1
#endif

   int      ComPort     = -1 ;            /* file descriptor for ttyS?   */
   int      ComOwnd     = FALSE ;         /* for CleanUp () ...          */
   int      ComFail     = TRUE ;          /* so ?? I am a pessimist !     */
   char     ComOride    = FALSE ;

   struct termios  otermios ;             /* original ttyS? settings   */
   struct termios  ntermios ;             /* altacc   ttyS? settings   */

   int    EchoIsOff = 0 ;                 /* for error recovery */
   struct termios  ottyios ;              /* original tty settings     */
   struct termios  nttyios ;              /* altacc   tty settings     */

   extern   char   * optarg ;             /* required for lib fun getarg */
   extern   int      optind ;             /* ditto                       */

   int        Verbose = TRUE ;
   char   * progname       ;              /* who am i, anyway ?          */
   char   * proghome       ;              /* where am i ???????          */

#define     ALTACC_LINE_MAX        8
#define     ALTACC_TALK_MAX     256

   typedef  struct   AltAccTalk
   {
      byte  A ;
      byte  P ;
   } AltAccTalk ;

   AltAccTalk   AltAccAry [ ALTACC_TALK_MAX+1 ] ;

#define ALTACC_BUFF_MAX    ALTACC_LINE_MAX * ALTACC_TALK_MAX + 2

   char   AltAccBuff [ ALTACC_BUFF_MAX + 8 ] ;

#define     NAME_LEN_MAX       256

   FILE   * OutFileAddr ;
   char     OutFileName [ NAME_LEN_MAX+1 ] ;

   char     ComFileName [ NAME_LEN_MAX+1 ] ;

   typedef  struct   CalData
   {
      char * Tag ;
      char * Nam ;
      double Val ;
   } CalData ;

   CalData AltAccData [ ] =
   {
      "actalt", "Actual Altitude",                0.0,
      "actbp",  "Actual Barometric Pressure",     0.0,
      "avgbp",  "AltAcc Pressure Avg",            0.0,
      "stdbp",  "AltAcc Pressure Std Dev",        0.0,
      "avg-1g", "Minus One Gee Avg",              0.0,
      "std-1g", "Minus One Gee Std Dev",          0.0,
      "fid(0)", "Finite Difference on [ -1, 0 ]", 0.0,
      "avg0g",  "Zero Gee Avg",                   0.0,
      "std0g",  "Zero Gee Std Dev",               0.0,
      "fid(1)", "Finite Difference on [ 0, 1 ]",  0.0,
      "avg+1g", "Plus One Gee Avg",               0.0,
      "std+1g", "Plus One Gee Std Dev",           0.0,
      "do/dg",  "Slope of AltAcc Output per G",   0.0,
      "y[0]",   "Y-Intercept of AltAcc Output",   0.0,
      "ccoff",  "Correlation Coefficient",        0.0
   } ;

#define NUM_ALTACC_DATA ( sizeof ( AltAccData ) /  ( sizeof ( CalData )))

#define      ActAlt     0
#define      ActBP      1
#define      AvgBP      2
#define      StDBP      3
#define      AvgNegG    4
#define      StDNegG    5
#define      FiDNegG    6
#define      AvgZeroG   7
#define      StDZeroG   8
#define      FiDZeroG   9
#define      AvgOneG   10
#define      StDOneG   11
#define      Slope     12
#define      YZero     13
#define      CCoff     14

#define     NAME_LEN_MAX       256

   FILE   * NitFileAddr ;
   char     NitFileName [ NAME_LEN_MAX+1 ] ;

#define  NIT_NAME    "prodata.nit"
#define  CAL_NAME    "prodata.cal"
#define  OUT_NAME    "prodata.dat"

   typedef  struct   NitData
   {
      char * Tag ;
      int    Ptr ;
   }  NitData ;

   NitData NitTags [ ] =
   {
      "none",   0,
      "port",   1,
      "time",   2,
      "alt",    3,
      "vel",    4,
      "acc",    5,
      "press",  6,
      "cal",    7,
      "xducer", 8,
   } ;

#define  NUM_NIT_TAGS   ( sizeof ( NitTags ) / sizeof ( NitData ))
#define  NT_PORT  1
#define  NT_TIME  2
#define  NT_ALT   3
#define  NT_VEL   4
#define  NT_ACC   5
#define  NT_PRESS 6
#define  NT_CAL      7
#define  NT_XDUCER   8

   void           Breaker ( int );
   void           SafeOut ( char * );
   int            SafeIn ( void );
   void           CleanUp ( void ) ;
   int            SetPort ( int, struct termios *, struct termios * ) ;
   void           Usage ( void ) ;
   char           YesNo ( char * ) ;
   void           Exit_YesNo ( char * ) ;
   void           Sleep ( double   ) ;
   u16            LoadAltAccAry ( AltAccTalk * ) ;
   u16            Parse ( char *, u16           , char * * ) ;
   void           GetaLoadaData ( u16           , u16            ) ;
   char         * ToLower ( char * ) ;
   char         * GetMyName ( char *, char * ) ;
   int            IsInList ( byte, char * ) ;
   char         * GetMyHome ( char *, char * ) ;
   void           Initialize ( void ) ;

   /* the damn compiler err'd out unless these were global */

   u16              N          [ 4 ] ;
   double           Sum        [ 4 ] ;
   double           SumSquares [ 4 ] ;

#define   TICK_CHAR              '.'
#define   COMMENT_CHAR           '#'
#define   DIR_SEP                '/'           /* change for unix / mac */
#define   ALTACC_DUMP_LEN        8196
#define   KEY_INP_MAX            256

/* ------------------------------------------------------------------------ */
main ( argc, argv )
/* ------------------------------------------------------------------------ */
  int      argc;
  char   * argv [];
/* ------------------------------------------------------------------------ */
{
   int c, l ;

   int o = 0 ;

   int i = 0 ;

   char  KeyInpBuf [ KEY_INP_MAX +  1 ] ;

   double           dtemp ;
   double           std_y_x ;
   double           std_y ;

   /* get the prog name from the opsys in there */

   progname = GetMyName ( progname, argv [0] ) ;
   proghome = GetMyHome ( proghome, argv [0] ) ;

   /* initialize the .rc file name now */

   sprintf ( NitFileName, "%s%c%s", proghome, DIR_SEP, NIT_NAME ) ;

   /* default action is a local copy of prodata.cal */

   strncpy ( OutFileName, CAL_NAME, NAME_LEN_MAX   ) ;

   /* default -- read the AltAcc on serial port Whatever ( ttyS0 or COM1 ) */

   strncpy ( ComFileName, DPORT, NAME_LEN_MAX   ) ;

   /* install [[Ctrl]+[C]] handler */

   if (( signal ( SIGINT, Breaker )) == SIG_ERR )
   {
      fprintf ( stderr, "\nprogram %s could not set Break Handler\n", progname );
      abort () ;
   }

   /* if we own an IRQ, we own responsibility for releasing it too */

   atexit ( CleanUp ) ;

   /* look for command line flags */

   while ( 1 )
   {
      o = getopt ( argc, argv, "p:o:n:qhv" ) ;

      if ( o == EOF )
        break ;

      switch (o)
      {
         case 'p':
            strncpy ( ComFileName, optarg, NAME_LEN_MAX );
            ComOride = TRUE ;
            break ;

         case 'n':
            strncpy ( NitFileName, optarg, NAME_LEN_MAX );
            continue ;

         case 'o':
            strncpy ( OutFileName, optarg, NAME_LEN_MAX );
            break ;

         case 'h':
            Usage () ;
            break ;

         case 'q':
            Verbose = FALSE ;
            break ;

         case 'v':
         case '?':
            Usage () ;
            break ;
      }
   }

   /* look for a nekked file name arg out there ( ignore multi names ) */

   if ( optind < argc )
      strncpy ( OutFileName, argv [ optind ], NAME_LEN_MAX );

   /* go read the .rc file */

   Initialize () ;

   /* Make sure the caller did not mess up the port id */

   if (( ComPort = open ( ComFileName, O_RDWR | O_NOCTTY )) < 0 )
   {
      fprintf ( stderr, "\n%s cannot open port %s\n", progname, ComFileName ) ;
      exit ( 1 );
   }

   /* if we own the com port we own responsibility for releasing it too */

   if ( ! ( ComOwnd = SetPort ( ComPort, & otermios, & ntermios )))
   {
      fprintf ( stderr, "\n%s cannot set AltAcc port for IO\n", progname ) ;
      exit ( 1 );
   }

   fprintf ( stderr, "%s is gathering calibration data from the AltAcc on %s\n",
                         progname, ComFileName ) ;

   fprintf ( stderr,
             "\nEnter the absolute Barometric Pressure ( enter x to exit ) " );

   fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

   if ( strlen ( KeyInpBuf ) > 1 )
   {
      if ( toupper ( * KeyInpBuf ) == 'X' )
         exit ( 3 );

      AltAccData [ ActBP ].Val = atof ( KeyInpBuf ) ;

      fprintf ( stderr,"Enter the actual altitude ( x to exit ) " );
      fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

      if ( toupper ( * KeyInpBuf ) == 'X' )
         exit ( 3 );

      AltAccData [ ActAlt ].Val = atof ( KeyInpBuf ) ;

      /* go get a load of Talk data */

      GetaLoadaData ( 1, 0 );

      AltAccData [ AvgBP ].Val = Sum [0] / N [0] ;
      AltAccData [ StDBP ].Val = sqrt (( SumSquares [0] - 
                                       ( Sum [0] * Sum [0] / N [0] ))
                                       / ( N [0] - 1 )) ;
   }

   fprintf ( stderr, "Set the AltAcc Upside Down to Measure -1 G\n" ) ;
   fprintf ( stderr, "then press enter when ready or x to quit    " );

   fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

   if ( toupper ( * KeyInpBuf ) == 'X' )
      exit ( 3 );

   /* go get a load of Talk data */

   GetaLoadaData ( 0, 1 );

   AltAccData [ AvgNegG ].Val = Sum [1] / N [1] ;
   AltAccData [ StDNegG ].Val = sqrt (( SumSquares [1] - 
                                      ( Sum [1] * Sum [1] / N [1] ))
                                      / ( N [1] - 1 )) ;

   fprintf ( stderr, "Set the AltAcc Flat to Measure Zero G\n" ) ;
   fprintf ( stderr, "then press enter when ready or x to quit    " );

   fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

   if ( toupper ( * KeyInpBuf ) == 'X' )
         exit ( 3 );

   /* go get a load of Talk data */

   GetaLoadaData ( 0, 2 );

   AltAccData [ AvgZeroG ].Val = Sum [2] / N [2] ;
   AltAccData [ StDZeroG ].Val = sqrt (( SumSquares [2] - 
                                       ( Sum [2] * Sum [2] / N [2] ))
                                       / ( N [2] - 1 )) ;

   AltAccData [ FiDNegG ].Val  = AltAccData [ AvgZeroG ].Val -
                                 AltAccData [ AvgNegG ].Val ;

   fprintf ( stderr, "Stand the AltAcc Rightside Up to Measure Plus One G\n" ) ;
   fprintf ( stderr, "then press enter when ready or x to quit    " );

   fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

   if ( toupper ( * KeyInpBuf ) == 'X' )
         exit ( 3 );

   /* go get a load of Talk data */

   GetaLoadaData ( 0, 3 );

   AltAccData [ AvgOneG ].Val = Sum [ 3 ] / N [3] ;
   AltAccData [ StDOneG ].Val = sqrt (( SumSquares [3] - 
                                      ( Sum [3] * Sum [3] / N [3] ))
                                      / ( N [3] - 1 )) ;

   AltAccData [ FiDZeroG ].Val  = AltAccData [ AvgOneG ].Val -
                                  AltAccData [ AvgZeroG ].Val ;

   /* calculate slope.  Least Squares is simplified with X = { -1,0,+1 } */

   AltAccData [ Slope ].Val = ( Sum[3] - Sum[1] ) / ( N[3] + N[1] ) ;

   /* Y-Intercept is the Avg G Value: */

   AltAccData [ YZero ].Val = ( Sum[1]+Sum[2]+Sum[3] ) / ( N[1]+N[2]+N[3] ) ;

   /* Correlation Coefficient = 1 - std^2_y_x / std_y^2 */

   /* Estimate Output at G = -1, do sum of diff squared  */

   dtemp =   AltAccData [ AvgNegG ].Val -
           ( -1 * AltAccData [ Slope ].Val + AltAccData [ YZero ].Val ) ;

   std_y_x = N[1] * N[1] * dtemp * dtemp ;

   /* Estimate Output at G = 0 */

   dtemp = AltAccData [ AvgZeroG ].Val - AltAccData [ YZero ].Val ;

   std_y_x += ( N[2] * N[2] * dtemp * dtemp ) ;

   /* Estimate Output at G = 1 */

   dtemp =   AltAccData [ AvgOneG ].Val -
           ( AltAccData [ Slope ].Val + AltAccData [ YZero ].Val ) ;

   std_y_x = ( N[3] * N[3] * dtemp * dtemp ) ;

   dtemp = Sum[1] + Sum[2] + Sum[3] ;
   N[0]  = N[1] + N[2] + N[3] ;

   std_y_x /= ( N[0] - 2 ) ;

   std_y   = (( SumSquares[1] + SumSquares[2] + SumSquares[3] ) -
             (( dtemp * dtemp ) / N[0] )) / ( N[0] - 1 ) ;

   AltAccData [ CCoff ].Val = 1.0 - ( std_y_x / std_y ) ;

   /* Test for proper operation and a good unit -- this code causes */
   /* an intermittent compiler error !!!! */


   if (( AltAccData [ FiDNegG ].Val <= 0.0 ) ||
       ( AltAccData [ FiDZeroG ].Val <= 0.0 ))
   {
      fprintf ( stderr, 
             "\n*** Warning *** Average Values indicate calibration error\n" ) ;
      fprintf ( stderr, 
             "                or a defective unit.  Save data (y|n) ?  " ) ;
      fgets ( KeyInpBuf, KEY_INP_MAX, stdin ) ;

      if ( toupper ( * KeyInpBuf ) != 'Y' )
         exit ( 3 );
   }

   /* Dump is in memory -- make sure I am allowed to open the Output File */

   if (( OutFileAddr = fopen ( OutFileName, "w" )) == NULL )
   {
         fprintf ( stderr, "\n%s cannot open file %s for output\n",
                              progname, OutFileName ) ;
         exit ( 4 );
    }

   fprintf ( stderr, "%s opened file %s for output\n",
                                progname, OutFileName ) ;
   fprintf ( OutFileAddr, "%c\n",COMMENT_CHAR );
   fprintf ( OutFileAddr, "%c AltAcc Calibration Data Compiled by %s\n",
                                 COMMENT_CHAR, progname );
   fprintf ( OutFileAddr, "%c\n",COMMENT_CHAR );

   for ( i = 0 ; i <  NUM_ALTACC_DATA ; i++ )
   {
      fprintf ( OutFileAddr, "%-6s  %10.4f  %c %s\n",
                  AltAccData [i].Tag,
                  AltAccData [i].Val,
                  COMMENT_CHAR ,
                  AltAccData [i].Nam ) ;

      if ( Verbose )
      {
         fprintf ( stderr, "%-6s  %10.4f  %c %s\n",
                            AltAccData [i].Tag,
                            AltAccData [i].Val,
                            COMMENT_CHAR ,
                            AltAccData [i].Nam ) ;
      }
   }

   fclose ( OutFileAddr ) ;               /* close the output file          */

   exit ( 0 ) ;

}
/* ------------------------------------------------------------------------ */
void  Usage ()
/* ------------------------------------------------------------------------ */
{
   fprintf ( stderr, 
   "%s -- Builds a calibration file for the AltAcc on the Com Port\n\n",
             progname ) ;
   fprintf ( stderr, 
   "Usage:  %s [-p COM#] [-n FILE] [-o FILE] [-q] -h  FILE\n",
                              progname );
   fprintf ( stderr, 
   "where:\n" );
   fprintf ( stderr, 
   "        -p COM# -> substitute { COM1, COM2, COM3, COM4 }\n" );
   fprintf ( stderr, 
   "        -n FILE -> override default init file prodata.nit with FILE\n" );
   fprintf ( stderr, 
   "        -o FILE -> legal local file name for the calibration data\n" );
   fprintf ( stderr, 
   "        -q      -> be quiet about it\n" );
   fprintf ( stderr, 
   "        -h      -> help ( this list )\n" );
   fprintf ( stderr, 
   "           FILE -> FILE == legal local file name for the calibration data\n" );
   fprintf ( stderr, 
   "                           ( same effect as the -o FILE option )\n" );

   exit ( 6 ) ;
}
/* ------------------------------------------------------------------------ */
int SetPort ( Port, OldTermIOS, NewTermIOS )
/* ------------------------------------------------------------------------ */
    int              Port ;
    struct termios * OldTermIOS ;
    struct termios * NewTermIOS ;
/* ------------------------------------------------------------------------ */
{
   tcgetattr ( Port, OldTermIOS ) ;                         /* Old settings */
   bzero ( NewTermIOS, sizeof ( struct termios ));          /* New settings */

   NewTermIOS->c_cflag = BAUD | CS8 | CREAD | CLOCAL ;
   NewTermIOS->c_iflag = IGNPAR;
   NewTermIOS->c_oflag = 0;

   /* set input mode (non-canonical, no echo, no signals) */

   NewTermIOS->c_lflag = 0 ;

   /* zero out all control signals */

   NewTermIOS->c_cc[VINTR]    = 0;     /* Ctrl-c                       */
   NewTermIOS->c_cc[VQUIT]    = 0;     /* Ctrl-\                       */
   NewTermIOS->c_cc[VERASE]   = 0;     /* del                          */
   NewTermIOS->c_cc[VKILL]    = 0;     /* @                            */
   NewTermIOS->c_cc[VEOF]     = 0;     /* Ctrl-d                       */
   NewTermIOS->c_cc[VTIME]    = 20;    /* inter-character timer==2 sec */
   NewTermIOS->c_cc[VMIN]     = 1;     /* blocking read - one char per */
   NewTermIOS->c_cc[VSWTC]    = 0;     /* '\0'                         */
   NewTermIOS->c_cc[VSTART]   = 0;     /* Ctrl-q                       */
   NewTermIOS->c_cc[VSTOP]    = 0;     /* Ctrl-s                       */
   NewTermIOS->c_cc[VSUSP]    = 0;     /* Ctrl-z                       */
   NewTermIOS->c_cc[VEOL]     = 0;     /* '\0'                         */
   NewTermIOS->c_cc[VREPRINT] = 0;     /* Ctrl-r                       */
   NewTermIOS->c_cc[VDISCARD] = 0;     /* Ctrl-u                       */
   NewTermIOS->c_cc[VWERASE]  = 0;     /* Ctrl-w                       */
   NewTermIOS->c_cc[VLNEXT]   = 0;     /* Ctrl-v                       */
   NewTermIOS->c_cc[VEOL2]    = 0;     /* '\0'                         */

   tcflush ( Port, TCIFLUSH );

   if ( tcsetattr ( Port, TCSANOW, NewTermIOS ) < 0 )
      return ( FALSE ) ;
   else
      return ( TRUE ) ;

}
/* ------------------------------------------------------------------------ */
void   CleanUp ()
/* ------------------------------------------------------------------------ */
{
   if ( ComOwnd )
      tcsetattr ( ComPort, TCSANOW, &otermios );

   if ( EchoIsOff )
      tcsetattr ( STDIN_FILENO, TCSANOW, &ottyios );

   /* atexit was exec'd after we took the [Ctrl][Break] IRQ ... so ...      */

   signal ( SIGINT, SIG_IGN );          /* and let dos do whatever it do    */
}
/* ------------------------------------------------------------------------ */
void  Breaker ( sig )
/* ------------------------------------------------------------------------ */
      int   sig ;
/* ------------------------------------------------------------------------ */
{
   /* A signal handler must take a single argument. The argument can be
    * tested within the handler and thus allows a single signal handler
    * to handle several different signals. In this case, the parameter
    * is included to keep the compiler from generating a warning but is
    * ignored because this signal handler only handles one interrupt:
    * SIGINT (Ctrl+C).
    */

   /* Disallow CTRL+C during handler. */

   signal ( SIGINT, SIG_IGN );

   if ( EchoIsOff )
      tcsetattr ( STDIN_FILENO, TCSANOW, &ottyios );

   Exit_YesNo ( "Break Detected - abort processing (y|n)? " );

   /* The CTRL+C interrupt must be reset to our handler since
    * by default it is reset to the system handler.
    */

   signal ( SIGINT, Breaker );
}
/* ------------------------------------------------------------------------ */
void  SafeOut ( str )         /* Outputs a string using system level calls. */
/* ------------------------------------------------------------------------ */
      char *str ;
/* ------------------------------------------------------------------------ */
{
   fputs ( str, stdout ) ;
}
/* ------------------------------------------------------------------------ */
int   SafeIn ()             /* Inputs a character using system level calls. */
/* ------------------------------------------------------------------------ */
{
   /* this program gets a char from stdin without any echo ( especially that
      pesky ole [Return] char that hoses up the pretty screen.  Note that 
      this lil beauty flushes all input after getting the char !           
    */

   int c = 0 ;

   tcgetattr ( STDIN_FILENO, & ottyios ) ;                  /* Old settings */
   memcpy ( & nttyios, & ottyios, sizeof ( nttyios ));      /* New == Old   */

   /* set input mode ( no echo ) */

   nttyios.c_lflag &= ~ICANON ;                           /* Just Grab char */
   nttyios.c_lflag &= ~ECHO ;                              /* turn off echo */

   EchoIsOff = TRUE ;                                 /* keep a global flag */
   tcsetattr ( STDIN_FILENO, TCSANOW, & nttyios ) ;

   c = fgetc ( stdin ) ;

   if ( EchoIsOff )
   {
      tcsetattr ( STDIN_FILENO, TCSANOW, & ottyios ) ;
      EchoIsOff = FALSE ;
   }
   return ( c ) ;
}
/* ------------------------------------------------------------------------ */
char YesNo ( Mess )
/* ------------------------------------------------------------------------ */
     char * Mess   ;
/* ------------------------------------------------------------------------ */
{
   int c ;
   char str [] = " " ;

   tcflush ( STDIN_FILENO, TCIFLUSH );  /* flush input so we get a good char */

   SafeOut ( Mess );                    /* ask dos to dump the message */

   c = SafeIn ();                       /* ask dos for an input char   */

   str [0] = c;                         /* make a string out of it ... */
   SafeOut ( str );                     /* and ask dos to dump it      */
   SafeOut ( "\n" );

   if ( isupper ( c ))
      c = tolower ( c ) ;

   return ( c ) ;
}
/* ------------------------------------------------------------------------ */
void Exit_YesNo ( Mess )
/* ------------------------------------------------------------------------ */
     char * Mess   ;
/* ------------------------------------------------------------------------ */
{
   if ( YesNo ( Mess ) == 'y' )
      exit ( 3 ) ;
}
/* ------------------------------------------------------------------------ */
void Sleep ( wait )
/* ------------------------------------------------------------------------ */
     double   wait ;
/* ------------------------------------------------------------------------ */
{
    time_t start, goal ;

    time ( & start ) ;

    do
    {
        time ( & goal ) ;

    } while (( difftime ( goal, start )) < wait ) ;
}
/* ------------------------------------------------------------------------ */
u16 LoadAltAccAry ( Ary )
/* ------------------------------------------------------------------------ */
    AltAccTalk    * Ary ;
/* ------------------------------------------------------------------------ */
{

#define  PARSE_MAX      3

   u16            i = 0 ;                    /* Array Cursor */
   u16            j = 0 ;                    /* Line Cursor  */

   int k = 0 ;

   char * BuffPtr = AltAccBuff ;
   char * TokPtr ;

   byte  c = 0 ;
   byte  l = 0 ;

   u16            NumArg ;
   char         * ParsePtr [ PARSE_MAX ] ;

   char  TalkCmd [] = "/T" ;

   write ( ComPort, TalkCmd, 2 ) ;
   tcflush ( ComPort, TCIOFLUSH );

   /* we are gonna load up a 256 * 8 + 2 char array here ... */

   while ( i < ALTACC_BUFF_MAX )
   {
      j = read ( ComPort, BuffPtr, 8 ) ;          /* AAA PPP\n == 8 chars */

      i += j ;
      k += j ;
      BuffPtr += j ;
 
      if ( k >= 48 )
      {
         putc ( TICK_CHAR, stderr );
         k = 0 ;
      }
   }

   if ( i > 2048 )
      i = 2048 ;

   tcflush ( ComPort, TCIOFLUSH );

   AltAccBuff [ i ] = '\0' ;
   BuffPtr = AltAccBuff ;

   i = 1 ;                     /* Number of good pairs in AltAccData[0].A */
   j = 0 ;                     /* Number of bad  pairs in AltAccData[0].P */

   TokPtr = strtok ( BuffPtr, "\n" ) ;    /* let the c-lib extract a line */

   while (( TokPtr != NULL ) && ( i < 256 ))
   {
      NumArg = Parse ( TokPtr, PARSE_MAX, ParsePtr ) ;

      if ( NumArg == 2 )
      {
         AltAccAry [i].A = (byte) atoi ( ParsePtr [0] );
         AltAccAry [i].P = (byte) atoi ( ParsePtr [1] );
      }
      else
         j ++ ;

      TokPtr = strtok ( NULL, "\n" );

      i ++ ;
   }

   AltAccAry [0].A = (byte) ( -- i ) ;
   AltAccAry [0].P = (byte) j ;

   fprintf ( stdout, "\n" ) ;

   return ( i + 1 ) ;
}
/* ------------------------------------------------------------------------ */
u16 Parse (line, maxarg, word)
/* ------------------------------------------------------------------------ */
    char            * line;
    u16               maxarg;
    char            * word [];
/* ------------------------------------------------------------------------ */

{
   u16               i = 0;
   char            * j = line + strlen (line);

   while (( line < j ) && ( i < maxarg ))
   {
      while (( line < j ) && ( isspace ( *line )))
         line++ ;

      if (( line >= j ) || ( *line == COMMENT_CHAR ))
         break;

      word [ i++ ] = line;

      while (( line < j ) && !( isspace ( *line )))
         line++ ;

      if ( line >= j )
         break;

      *line++ = '\0';
  }

  return ( i ) ;

}
/* ------------------------------------------------------------------------ */
void  GetaLoadaData ( what, ndx )
/* ------------------------------------------------------------------------ */
      u16               what;
      u16               ndx;
/* ------------------------------------------------------------------------ */
{
   u16               i ;
   double            dtemp ;
   char              c ;

   while ( 1 )
   {
       i = LoadAltAccAry ( AltAccAry ) ;

      fprintf ( stderr,"%s read %d lines from the AltAcc on %s\n",
                        progname, i, ComFileName );

      c = YesNo ( "Accept AltAcc data ( y-yes | n-no | x-exit ) ? " ) ;

      if ( c == 'x' )
         exit ( 3 );
      else if ( c == '\n' )
          c = ' ' ;
      else if ( c == '\r' )
          c = ' ' ;

      if ( c != 'n' )         /* i.e. default answer == 'y' */
         break ;
   }

   N [ndx] = AltAccAry [0].A ;
   Sum [ndx ] = 0.0 ;
   SumSquares [ndx] = 0.0 ;

   i = 1;

   while ( i <= N [ndx] )
   {
      if ( what == 0 )
         dtemp = AltAccAry [i].A ;
      else
         dtemp = AltAccAry [i].P ;

      Sum [ndx] += dtemp ;

      SumSquares [ndx] += ( dtemp * dtemp ) ;

      i ++ ;
   }
}
/* ------------------------------------------------------------------------ */
char   * ToLower ( InStr )
/* ------------------------------------------------------------------------ */
   char * InStr ;
/* ------------------------------------------------------------------------ */
{
   u16              i = 0 ;

   while ( i < strlen ( InStr ))
   {
      if ( isupper ( InStr [ i ] ))
         InStr [ i ]  = tolower ( InStr [ i ] ) ;

      i ++ ;
   }

   return ( InStr ) ;

}
/* ------------------------------------------------------------------------ */
char   * GetMyName ( Name,  ArgV )
/* ------------------------------------------------------------------------ */
   char * Name ;
   char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;

   /* Throw out the path */

   if (( Name = strrchr ( ArgV, DIR_SEP )) != NULL )
   {
      * Name = '\0' ;
        Name ++ ;
   }
   else
      Name = ArgV ;

   /* throw away the '.exe.' */

   if (( p = strrchr ( Name, '.' )) != NULL )
      * p = '\0' ;

   return ( Name ) ;

}
/* ------------------------------------------------------------------------ */
int  IsInList ( Char, List )
/* ------------------------------------------------------------------------ */
      byte   Char ;
      char * List ;
/* ------------------------------------------------------------------------ */
{
   int i = 0 ;

   for ( i = 0 ; i < strlen ( List ) ; i ++ )
      if ( Char == List [i] )
         return ( 1 ) ;

   return ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
char  * GetMyHome ( Path,  ArgV )
/* ------------------------------------------------------------------------ */
      char * Path ;
      char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;
   int    i = 0 ;
   int    j = 0 ;

   /* First look for a Home for the System Environment */

   if (( p = getenv ( "PRODATA" )) != NULL )
      return ( p ) ;

   /* None found, try for a fqpn */

   Path = ArgV ;

   p = ArgV + strlen ( ArgV ) - 1;

   while ( p >= Path )
   {
      if ( IsInList ( * p, "/\\" ))
         * p = '\0' ;
      else
         break ;

      p -- ;
   }

   /* strip off /..'s  and count the occurances */

   while ( 1 )
   {
      i = strlen ( Path ) - 1 ;

      if (( i > 1 ) && ( Path [i] == '.' ) &&
                       ( Path [i-1] == '.' ) &&
                       ( Path [i-2] == DIR_SEP ))
      {
         Path [i-2] = '\0' ;
         j ++ ;
      }
      else
         break ;
   }

   /* now trim the corresponding dir's  */

   i = 0 ;

   while ( i < j )
   {
      if (( p = strrchr ( Path, DIR_SEP )) != NULL )
            * p = '\0' ;
      i ++ ;
   }

   /* a unix system puts the program name in the env sans path, so ... */

   if ( strcmp ( Path, progname ) != 0 )
      return ( Path ) ;
   else
      return ( "." ) ;

}
/* ------------------------------------------------------------------------ */
void  Initialize ( )
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;
   u16   j = 0 ;

#ifndef INP_SIZE
   #define INP_SIZE    80
   #define MAXARG       3
#endif

   char     InpBuf [ INP_SIZE+1 ] ;
   char  *  ArgBuf [ MAXARG ] ;

   u16      NumArg = 0;

   if (( NitFileAddr = fopen ( NitFileName, "rb" )) == NULL )
   {
      fprintf ( stderr, "\n%s cannot initialization file %s for input\n",
                           progname, NitFileName ) ;
   }
   else
   {
      while ( ! feof ( NitFileAddr ))
      {
         if ( fgets ( InpBuf, INP_SIZE, NitFileAddr ) != NULL )
         {
            if ( Parse ( InpBuf, MAXARG, ArgBuf ) > 1 )
            {
               ToLower ( ArgBuf [0] ) ;

               for ( i = 0 ; i < NUM_NIT_TAGS ; i ++ )
                  if ( strcmp ( NitTags [i].Tag, ArgBuf [0] ) == 0 )
                     break ;

               if ( i < NUM_NIT_TAGS )
               {
                  switch ( NitTags [i].Ptr )
                  {
                     case NT_PORT:

                        if ( ! ComOride )
                           strncpy ( ComFileName, ArgBuf [1], NAME_LEN_MAX );

                        break ;

                     default:
                        break ;
                  }
               }
            }
         }
      }
      fclose ( NitFileAddr ) ;
   }
}
void FooFunct ()
{
   tcgetattr ( ComPort, &otermios ) ;        /* Original tty settings */
   bzero ( &ntermios, sizeof ( ntermios ));  /* AltAcc   tty settings */

   ntermios.c_cflag = BAUD | CS8 | CREAD | CLOCAL ;
   ntermios.c_iflag = IGNPAR;
   ntermios.c_oflag = 0;

   /* set input mode (non-canonical, no echo, no signals) */

   ntermios.c_lflag = 0 ;

   /* zero out all control signals */

   ntermios.c_cc[VINTR]    = 0;     /* Ctrl-c                       */
   ntermios.c_cc[VQUIT]    = 0;     /* Ctrl-\                       */
   ntermios.c_cc[VERASE]   = 0;     /* del                          */
   ntermios.c_cc[VKILL]    = 0;     /* @                            */
   ntermios.c_cc[VEOF]     = 0;     /* Ctrl-d                       */
   ntermios.c_cc[VTIME]    = 0;     /* inter-character timer unused */
   ntermios.c_cc[VMIN]     = 1;     /* blocking read                */
   ntermios.c_cc[VSWTC]    = 0;     /* '\0'                         */
   ntermios.c_cc[VSTART]   = 0;     /* Ctrl-q                       */
   ntermios.c_cc[VSTOP]    = 0;     /* Ctrl-s                       */
   ntermios.c_cc[VSUSP]    = 0;     /* Ctrl-z                       */
   ntermios.c_cc[VEOL]     = 0;     /* '\0'                         */
   ntermios.c_cc[VREPRINT] = 0;     /* Ctrl-r                       */
   ntermios.c_cc[VDISCARD] = 0;     /* Ctrl-u                       */
   ntermios.c_cc[VWERASE]  = 0;     /* Ctrl-w                       */
   ntermios.c_cc[VLNEXT]   = 0;     /* Ctrl-v                       */
   ntermios.c_cc[VEOL2]    = 0;     /* '\0'                         */

   tcflush ( ComPort, TCIFLUSH );
   tcsetattr ( ComPort, TCSANOW, &ntermios );

}
