/*
   This program is to clear the AltAcc and That is All
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>
#include <time.h>

#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include "getopt.h"

#define      TRUE            1
#define      FALSE           0

   typedef unsigned char   byte ;
   typedef unsigned short  u16 ;

#ifdef DOS
#define DPORT                "COM1"
#define BAUD                 B9600
#else
#define DPORT                "/dev/ttyS0"
#define BAUD                 B9600
#define _POSIX_SOURCE        1
#endif

   int      ComPort    = -1 ;           /* global comport               */
   int      ComOwnd    = FALSE ;        /* for CleanUp () ...           */
   int      ComFail    = TRUE ;         /* so ?? I am a pessimist !     */
   int      ComOride   = FALSE ;        /* command line override of .rc */

   struct termios  otermios ;                /* original tty settings     */
   struct termios  ntermios ;                /* altacc   tty settings     */

   int    EchoIsOff = 0 ;                 /* for error recovery */
   struct termios  ottyios ;              /* original tty settings     */
   struct termios  nttyios ;              /* altacc   tty settings     */

   extern   char  * optarg ;            /* required for lib fun getarg  */
   extern int     optind ;              /* ditto                        */

   int            Verbose = TRUE ;
   char *         progname ;            /* who am i, anyway ?           */
   char *         proghome ;            /* where am i ???               */

#define     NAME_LEN_MAX       256

   FILE   * NitFileAddr ;
   char     NitFileName [ NAME_LEN_MAX+1 ] ;
   char     ComFileName [ NAME_LEN_MAX+1 ] ;

#define  NIT_NAME    "prodata.nit"
#define  CAL_NAME    "prodata.cal"
#define  OUT_NAME    "prodata.dat"

   typedef  struct   NitData
   {
      char * Tag ;
      int    Ptr ;
   }  NitData ;

   NitData NitTags [ ] =
   {
      "none",   0,
      "port",   1,
      "time",   2,
      "alt",    3,
      "vel",    4,
      "acc",    5,
      "press",  6,
      "cal",    7,
      "xducer", 8,
   } ;

#define  NUM_NIT_TAGS   ( sizeof ( NitTags ) / sizeof ( NitData ))
#define  NT_PORT  1
#define  NT_TIME  2
#define  NT_ALT   3
#define  NT_VEL   4
#define  NT_ACC   5
#define  NT_PRESS 6
#define  NT_CAL      7
#define  NT_XDUCER   8

   void           Breaker ( int );
   void           SafeOut ( char * );
   int            SafeIn ( void );
   void           CleanUp ( void ) ;
   void           Usage ( void ) ;
   void           Sleep ( double   ) ;
   char           YesNo ( char * ) ;
   void           Exit_YesNo ( char * ) ;
   u16            Parse ( char *, u16, char * * ) ;
   char         * ToLower ( char * ) ;
   int            IsInList ( byte, char * ) ;
   char         * GetMyName ( char *, char * ) ;
   char         * GetMyHome ( char *, char * ) ;
   void           Initialize ( void ) ;

#define     CLEAR_TIME     55
#define     TICK_CHAR      '.'
#define     COMMENT_CHAR   '#'
#define     DIR_SEP        '/'

/* ------------------------------------------------------------------------ */
main ( argc, argv )
/* ------------------------------------------------------------------------ */
  int      argc;
  char   * argv [];
/* ------------------------------------------------------------------------ */
{

   int   c, l, t;

   int   AskFirst = TRUE ;

   char ClearCmd [] = "/CC" ;

   /* get the prog name from the opsys in there */

   progname = GetMyName ( progname, argv [0] ) ;
   proghome = GetMyHome ( proghome, argv [0] ) ;

   /* initialize the .rc file name now */

   sprintf ( NitFileName, "%s%c%s", proghome, DIR_SEP, NIT_NAME ) ;

   /* default -- read the AltAcc on serial port Whatever ( ttyS0 or COM1 ) */

   strncpy ( ComFileName, DPORT, NAME_LEN_MAX   ) ;

   /* install [[Ctrl]+[C]] handler */

   if (( signal ( SIGINT, Breaker )) == SIG_ERR )
   {
      fprintf ( stderr, "\nprogram %s could not set Break Handler\n", progname );
      abort () ;
   }

   /* if we own the interrupts we own responsibility for releasing em too */

   atexit ( CleanUp ) ;

   /* look for command line flags */

   while (( c = getopt ( argc, argv, "p:n:qhy" )) != EOF )
   {
      switch (c)
      {
         case 'p':
            strncpy ( ComFileName, optarg, NAME_LEN_MAX );
            ComOride = TRUE ;
            continue ;

         case 'n':
            strncpy ( NitFileName, optarg, NAME_LEN_MAX );
            continue ;

         case 'h':
            Usage () ;
            continue ;

         case 'q':
            Verbose = FALSE ;
            continue ;

         case 'y':
            AskFirst = FALSE ;
            continue ;

         case '?':
            continue ;
      }
   }

   /* This is a dangerous little prog -- Ask the user if he
      really meant to erase that M1939 flite before actually
      doing the data ;-)
    */

   if ( AskFirst )
      if ( 
      YesNo ( "\nDo you really mean to clear all data from the AltAcc (y|n) ? "           ) != 'y' )
      {
         exit ( 3 ) ;
      }

   /* go read the .rc file */

   Initialize () ;

   /* from this point on, CleanUp () has to run AsyncStop () */

   /* Make sure the caller did not mess up the port id */

   if (( ComPort = open ( ComFileName, O_RDWR | O_NOCTTY )) < 0 )
   {
      fprintf ( stderr, "\n%s cannot open port %s\n", progname, ComFileName ) ;
      exit ( 1 );
   }

   /* Take over the com port */

   tcgetattr ( ComPort, &otermios ) ;        /* Original tty settings */
   bzero ( &ntermios, sizeof ( ntermios ));  /* AltAcc   tty settings */

   ntermios.c_cflag = BAUD | CS8 | CREAD | CLOCAL ;
   ntermios.c_iflag = IGNPAR;
   ntermios.c_oflag = 0;

   /* set input mode (non-canonical, no echo,...) */
   ntermios.c_lflag = 0;

   ntermios.c_cc [ VTIME ] = 0 ;   /* inter-character timer ... no timeout    */
   ntermios.c_cc [ VMIN ]  = 0 ;   /* no min number of chars let read() do it */

   tcflush ( ComPort, TCIFLUSH );
   tcsetattr ( ComPort, TCSANOW, &ntermios );

   /* if we own the com port we own responsibility for releasing it too */

   ComOwnd = TRUE ;

   if ( Verbose )
   {
      fprintf ( stderr, "%s is clearing the AltAcc on %s\n",
                progname, ComFileName ) ;
   }

   write ( ComPort, ClearCmd, 3 ) ;
   tcflush ( ComPort, TCIOFLUSH );

   t = 0 ;                          /* initialize the timer */

   while ( t < CLEAR_TIME )
   {
      if ( Verbose )
         putc ( TICK_CHAR, stderr );

      Sleep ( 1.0 ) ;

      t ++ ;
   }

   if ( Verbose )
      fprintf ( stderr, "\n" ) ;

   exit ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
void  Usage ()
/* ------------------------------------------------------------------------ */
{
   fprintf ( stderr, "%s -- Clears ALL flight data stored in AltAcc EEProm\n\n",
             progname ) ;
   fprintf ( stderr, "Usage:  %s [ -p COM# ][ -n FILE ][ -q ] [ -y ] -h\n",
             progname );
   fprintf ( stderr, "where:\n" );
   fprintf ( stderr, "        -p COM# -> substitute { COM1, COM2, COM3, COM4 }\n" );
   fprintf ( stderr, "        -n FILE -> override default init file prodata.nit with FILE\n" );
   fprintf ( stderr, "        -q      -> be quiet about it\n" );
   fprintf ( stderr, "        -y      -> don't ask before clearing data\n" );
   fprintf ( stderr, "        -h      -> help ( this list )\n" );

   exit ( 6 ) ;
}
/* ------------------------------------------------------------------------ */
void   CleanUp ()
/* ------------------------------------------------------------------------ */
{
   if ( ComOwnd )
      tcsetattr ( ComPort, TCSANOW, &otermios );

   if ( EchoIsOff )
      tcsetattr ( STDIN_FILENO, TCSANOW, &ottyios );

   /* atexit () called after successfully stealing the [Ctrl][Break] IRQ   */

   signal ( SIGINT, SIG_IGN );            /* and let dos do whatever it do */

}
/* ------------------------------------------------------------------------ */
void  Breaker ( sig )
/* ------------------------------------------------------------------------ */
      int   sig ;
/* ------------------------------------------------------------------------ */
{
   /* A signal handler must take a single argument. The argument can be
    * tested within the handler and thus allows a single signal handler
    * to handle several different signals. In this case, the parameter
    * is included to keep the compiler from generating a warning but is
    * ignored because this signal handler only handles one interrupt:
    * SIGINT (Ctrl+C).
    */

   int c;
   char str[] = " ";

   /* Disallow CTRL+C during handler. */

   signal ( SIGINT, SIG_IGN );

   if ( EchoIsOff )
      tcsetattr ( STDIN_FILENO, TCSANOW, &ottyios );

   Exit_YesNo ( "Break Detected - abort processing (y|n)? " );

   signal ( SIGINT, Breaker );
}
/* ------------------------------------------------------------------------ */
void   SafeOut ( str )         /* Outputs a string using system level calls. */
/* ------------------------------------------------------------------------ */
      char *str ;
/* ------------------------------------------------------------------------ */
{
   fputs ( str, stdout ) ;
}
/* ------------------------------------------------------------------------ */
int   SafeIn ()             /* Inputs a character using system level calls. */
/* ------------------------------------------------------------------------ */
{
   /* this program gets a char from stdin without any echo ( especially that
      pesky ole [Return] char that hoses up the pretty screen.  Note that 
      this lil beauty flushes all input after getting the char !           
    */

   int c = 0 ;

   tcgetattr ( STDIN_FILENO, & ottyios ) ;                  /* Old settings */
   memcpy ( & nttyios, & ottyios, sizeof ( nttyios ));      /* New == Old   */

   /* set input mode ( no echo ) */

   nttyios.c_lflag &= ~ICANON ;                           /* Just Grab char */
   nttyios.c_lflag &= ~ECHO ;                              /* turn off echo */

   EchoIsOff = TRUE ;                                 /* keep a global flag */
   tcsetattr ( STDIN_FILENO, TCSANOW, & nttyios ) ;

   c = fgetc ( stdin ) ;

   if ( EchoIsOff )
   {
      tcsetattr ( STDIN_FILENO, TCSANOW, & ottyios ) ;
      EchoIsOff = FALSE ;
   }
   return ( c ) ;
}
/* ------------------------------------------------------------------------ */
void Sleep ( wait )
/* ------------------------------------------------------------------------ */
     double wait ;
/* ------------------------------------------------------------------------ */
{
   time_t start, goal ;

   time ( & start ) ;

   do
   {
      time ( & goal ) ;

   } while (( difftime ( goal, start )) < wait ) ;
}
/* ------------------------------------------------------------------------ */
char  YesNo ( Mess )
/* ------------------------------------------------------------------------ */
      char *  Mess   ;
/* ------------------------------------------------------------------------ */
{
   int c ;
   char str [] = " " ;

   tcflush ( STDIN_FILENO, TCIFLUSH );  /* flush input so we get a good char */

   SafeOut ( Mess );             /* ask dos to dump the message */

   c = SafeIn ();                /* ask dos for an input char   */
   str [0] = c;                  /* make a string out of it ... */
   SafeOut ( str );              /* and ask dos to dump it      */
   SafeOut ( "\n" );

   if ( isupper ( c ))
      c = tolower ( c ) ;

   return ( c ) ;
}
/* ------------------------------------------------------------------------ */
void   Exit_YesNo ( Mess )
/* ------------------------------------------------------------------------ */
      char * Mess   ;
/* ------------------------------------------------------------------------ */
{
   char c ;

   c = YesNo ( Mess ) ;

   if ( c == 'y' )
      exit ( 3 ) ;
}
/* ------------------------------------------------------------------------ */
u16   Parse (line, maxarg, word)
/* ------------------------------------------------------------------------ */
      char * line;
      u16    maxarg;
      char * word [];
/* ------------------------------------------------------------------------ */
{
   u16      i = 0;
   char   * j = line + strlen (line);

   while (( line < j ) && ( i < maxarg ))
   {
      while (( line < j ) && ( isspace ( *line )))
         line++ ;

      if (( line >= j ) || ( *line == COMMENT_CHAR ))
         break;

      word [ i++ ] = line;

      while (( line < j ) && !( isspace ( *line )))
         line++ ;

      if ( line >= j )
         break;

      *line++ = '\0';
  }

  return ( i ) ;

}
/* ------------------------------------------------------------------------ */
char   * ToLower ( InStr )
/* ------------------------------------------------------------------------ */
   char * InStr ;
/* ------------------------------------------------------------------------ */
{
   unsigned short   i = 0 ;

   while ( i < strlen ( InStr ))
   {
      if ( isupper ( InStr [ i ] ))
         InStr [ i ]  = tolower ( InStr [ i ] ) ;

      i ++ ;
   }

   return ( InStr ) ;

}
/* ------------------------------------------------------------------------ */
char   * GetMyName ( Name,  ArgV )
/* ------------------------------------------------------------------------ */
   char * Name ;
   char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;

   /* Throw out the path */

   if (( Name = strrchr ( ArgV, DIR_SEP )) != NULL )
   {
      * Name = '\0' ;
        Name ++ ;
   }
   else
      Name = ArgV ;

   /* throw away the '.exe.' */

   if (( p = strrchr ( Name, '.' )) != NULL )
      * p = '\0' ;

   return ( Name ) ;

}
/* ------------------------------------------------------------------------ */
int  IsInList ( Char, List )
/* ------------------------------------------------------------------------ */
      byte   Char ;
      char * List ;
/* ------------------------------------------------------------------------ */
{
   int i = 0 ;

   for ( i = 0 ; i < strlen ( List ) ; i ++ )
      if ( Char == List [i] )
         return ( 1 ) ;

   return ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
char  * GetMyHome ( Path,  ArgV )
/* ------------------------------------------------------------------------ */
      char * Path ;
      char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;
   int    i = 0 ;
   int    j = 0 ;

   /* First look for a Home for the System Environment */

   if (( p = getenv ( "PRODATA" )) != NULL )
      return ( p ) ;

   /* None found, try for a fqpn */

   Path = ArgV ;

   p = ArgV + strlen ( ArgV ) - 1;

   while ( p >= Path )
   {
      if ( IsInList ( * p, "/\\" ))
         * p = '\0' ;
      else
         break ;

      p -- ;
   }

   /* strip off /..'s  and count the occurances */

   while ( 1 )
   {
      i = strlen ( Path ) - 1 ;

      if (( i > 1 ) && ( Path [i] == '.' ) &&
                       ( Path [i-1] == '.' ) &&
                       ( Path [i-2] == DIR_SEP ))
      {
         Path [i-2] = '\0' ;
         j ++ ;
      }
      else
         break ;
   }

   /* now trim the corresponding dir's  */

   i = 0 ;

   while ( i < j )
   {
      if (( p = strrchr ( Path, DIR_SEP )) != NULL )
            * p = '\0' ;
      i ++ ;
   }

   /* a unix system puts the program name in the env sans path, so ... */

   if ( strcmp ( Path, progname ) != 0 )
      return ( Path ) ;
   else
      return ( "." ) ;

}
/* ------------------------------------------------------------------------ */
void  Initialize ( )
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;
   u16   j = 0 ;

#ifndef INP_SIZE
   #define INP_SIZE    80
   #define MAXARG       3
#endif

   char     InpBuf [ INP_SIZE+1 ] ;
   char  *  ArgBuf [ MAXARG ] ;

   u16      NumArg = 0;

   if (( NitFileAddr = fopen ( NitFileName, "rb" )) == NULL )
   {
      fprintf ( stderr, "\n%s cannot initialization file %s for input\n",
                           progname, NitFileName ) ;
   }
   else
   {
      while ( ! feof ( NitFileAddr ))
      {
         if ( fgets ( InpBuf, INP_SIZE, NitFileAddr ) != NULL )
         {
            if ( Parse ( InpBuf, MAXARG, ArgBuf ) > 1 )
            {
               ToLower ( ArgBuf [0] ) ;

               for ( i = 0 ; i < NUM_NIT_TAGS ; i ++ )
                  if ( strcmp ( NitTags [i].Tag, ArgBuf [0] ) == 0 )
                     break ;

               if ( i < NUM_NIT_TAGS )
               {
                  switch ( NitTags [i].Ptr )
                  {
                     case NT_PORT:

                        if ( ! ComOride )
                           strncpy ( ComFileName, ArgBuf [1], NAME_LEN_MAX );

                        break ;

                     default:
                        break ;
                  }
               }
            }
         }
      }
      fclose ( NitFileAddr ) ;
   }
}
