/*
   This program is to read the AltAcc 
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>
#include <time.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include "getopt.h"

#define       TRUE           1
#define       FALSE          0

   typedef unsigned char   byte ;
   typedef unsigned short  u16 ;

#ifdef DOS
#define DPORT                "COM1"
#define BAUD                 B9600
#else
#define DPORT                "/dev/ttyS0"
#define BAUD                 B9600
#define _POSIX_SOURCE        1
#endif

   int      ComPort     = -1 ;               /* file descriptor for ttyS? */
   int      ComOwnd     = FALSE ;            /* for CleanUp () ...        */
   int      ComFail     = TRUE ;             /* so ?? I am a pessimist !  */
   int      ComOride    = FALSE ;

   struct termios  otermios ;                /* original tty settings     */
   struct termios  ntermios ;                /* altacc   tty settings     */

   extern   char   * optarg ;                /* required for lib fun getarg */
   extern   int      optind ;                /* ditto                       */

   int      Verbose = TRUE ;
   char   * progname ;                      /* who am i, anyway ?          */
   char   * proghome ;                      /* who am i, anyway ?          */

   typedef  struct   NitData
   {
      char * Tag ;
      int    Ptr ;
   }  NitData ;

   NitData NitTags [ ] =
   {
      "none",   0,
      "port",   1,
      "time",   2,
      "alt",    3,
      "vel",    4,
      "acc",    5,
      "press",  6,
      "cal",    7,
      "xducer", 8,
   } ;

#define  NUM_NIT_TAGS   ( sizeof ( NitTags ) / sizeof ( NitData ))
#define  NT_PORT     1
#define  NT_TIME     2
#define  NT_ALT      3
#define  NT_VEL      4
#define  NT_ACC      5
#define  NT_PRESS    6
#define  NT_CAL      7
#define  NT_XDUCER   8

#define  BUFF_SIZE   8200

   byte   InpBuffer [ BUFF_SIZE ] ;
   byte   TmpBuffer [ BUFF_SIZE ] ;

#define  NAME_LEN_MAX 256

   FILE     * OutFileAddr ;
   char       OutFileName [ NAME_LEN_MAX+1 ] ;

   FILE   * NitFileAddr ;
   char     NitFileName [ NAME_LEN_MAX+1 ] ;

   char     ComFileName [ NAME_LEN_MAX+1 ] ;

#define  NIT_NAME    "prodata.nit"
#define  CAL_NAME    "prodata.cal"
#define  OUT_NAME    "prodata.dat"

#define   TICK_CHAR              0xFE
#define   COMMENT_CHAR           '#'
#define   DIR_SEP                '/'            /* change for unix / mac */
#define   ALTACC_DUMP_LEN        8196
#define   KEY_INP_MAX            256

   void     Breaker ( int );
   void     SafeOut ( char * );
   int      SafeIn ( void );
   void     CleanUp ( void ) ;
   void     Usage ( void ) ;
   char     YesNo ( char * ) ;
   void     Exit_YesNo ( char * ) ;
   void     Sleep ( double   ) ;
   u16      Parse ( char *, u16, char * * ) ;
   char   * ToLower ( char * ) ;
   int      IsInList ( byte, char * ) ;
   char   * GetMyName ( char *, char * ) ;
   char   * GetMyHome ( char *, char * ) ;
   void     Initialize ( void ) ;

/* ------------------------------------------------------------------------ */
main ( argc, argv )
/* ------------------------------------------------------------------------ */
  int      argc;
  char   * argv [];
/* ------------------------------------------------------------------------ */
{
   int c, l ;

   int o = 0 ;

   int i = 0 ;

   u16 ChkSum = 0 ;
   u16 AltAccSum = 0 ;
   u16 Boo = 0 ;

   char  * p ;

   char ReadCmd [] = "/R" ;

   /* get the prog name from the opsys in there */

   progname = GetMyName ( progname, argv [0] ) ;
   proghome = GetMyHome ( proghome, argv [0] ) ;

   /* initialize the .rc file name now */

   sprintf ( NitFileName, "%s%c%s", proghome, DIR_SEP, NIT_NAME ) ;

   /* default -- put the output in __this__ directory */

   strncpy ( OutFileName, OUT_NAME, NAME_LEN_MAX   ) ;

   /* default -- read the AltAcc on serial port Whatever ( ttyS0 or COM1 ) */

   strncpy ( ComFileName, DPORT, NAME_LEN_MAX   ) ;

   /* install [[Ctrl]+[C]] handler */

   if (( signal ( SIGINT, Breaker )) == SIG_ERR )
   {
      fprintf (stderr, "\nprogram %s could not set Break Handler\n", progname );
      abort () ;
   }

   /* if we own an IRQ, we own responsibility for releasing it too */

   atexit ( CleanUp ) ;

   /* look for command line flags */

   while ( 1 )
   {
      o = getopt ( argc, argv, "p:o:n:vhq" ) ;

      if ( o == EOF )
         break ;

      switch (o)
      {
         case 'p':
            strncpy ( ComFileName, optarg, NAME_LEN_MAX );
            ComOride = TRUE ;
            break ;

         case 'o':
            strncpy ( OutFileName, optarg, NAME_LEN_MAX );
            break ;

         case 'n':
            strncpy ( NitFileName, optarg, NAME_LEN_MAX );
            break ;

         case 'h':
            Usage () ;
            break ;

         case 'q':
            Verbose = FALSE ;
            break ;

         case 'v':
         case '?':
            Usage () ;
            break ;
      }
   }

   /* look for a nekked file name arg out there ( ignore multi names ) */

   if ( optind < argc )
      strncpy ( OutFileName, argv [ optind ], NAME_LEN_MAX );

   /* go read the .rc file */

   if ( Verbose )
      fprintf ( stderr, "reading %s\n", NitFileName ) ;

   Initialize () ;

   /* Make sure the caller did not mess up the port id */

   if (( ComPort = open ( ComFileName, O_RDWR | O_NOCTTY )) < 0 )
   {
      fprintf ( stderr, "\n%s cannot open port %s\n", progname, ComFileName ) ;
      exit ( 1 );
   }

   /* Take over the com port */

   tcgetattr ( ComPort, &otermios ) ;        /* Original tty settings */
   bzero ( &ntermios, sizeof ( ntermios ));  /* AltAcc   tty settings */

   ntermios.c_cflag = BAUD | CS8 | CREAD | CLOCAL ;
   ntermios.c_iflag = IGNPAR;
   ntermios.c_oflag = 0;
  
   /* set input mode (non-canonical, no echo,...) */
   ntermios.c_lflag = 0;
  
   ntermios.c_cc [ VTIME ] = 0 ;   /* inter-character timer ... no timeout    */
   ntermios.c_cc [ VMIN ]  = 0 ;   /* no min number of chars let read() do it */
  
   tcflush ( ComPort, TCIFLUSH );
   tcsetattr ( ComPort, TCSANOW, &ntermios );

   /* if we own the com port we own responsibility for releasing it too */

   ComOwnd = TRUE   ;

   /* initialize port to AltAcc Settings */

ProReadLoop:

   if ( Verbose )
   {
      fprintf ( stderr,"%s is reading the AltAcc on %s into %s\n",
                progname, ComFileName, OutFileName ) ;
      fprintf ( stderr, "\r%5d of %d bytes", 0, ALTACC_DUMP_LEN ) ;
   }

   i = c = 0 ;                /* buffer cursor */

   p = InpBuffer ;

   ChkSum = 0 ;               /* reinitialize for retries */

   write ( ComPort, ReadCmd, 2 ) ;
   tcflush ( ComPort, TCIOFLUSH );

   while ( i < ALTACC_DUMP_LEN )
   {
      c = read ( ComPort, TmpBuffer, ALTACC_DUMP_LEN ) ;
      memcpy ( p, TmpBuffer, c ) ;
      
      i += c ;
      p += c ;

      if ( Verbose )
         fprintf ( stderr, "\r%5d", i ) ;
   }
   
   if ( i > ALTACC_DUMP_LEN )
      i = ALTACC_DUMP_LEN ;

   if ( Verbose )
      fprintf ( stderr, "\n" ) ;

   /* Dump is in memory -- make sure I am allowed to open the Output File */

   if (( OutFileAddr = fopen ( OutFileName, "wb" )) == NULL )
   {
      fprintf ( stderr, "\n%s cannot open file %s for output\n",
                progname, OutFileName ) ;
      exit ( 4 );
   }

   AltAccSum = (( InpBuffer [ i-3 ] << 8 ) | InpBuffer [ i-4 ] ) ;

   c = i - 4 ;

   for ( o = 0 ; o < i ; o ++ )
   {
      fputc ( InpBuffer [ o ], OutFileAddr ) ;

      if ( o < c )
         ChkSum += ( u16 ) InpBuffer [ o ] ;
   }

   fflush ( OutFileAddr ) ;               /* flush the output file          */
   fclose ( OutFileAddr ) ;               /* close the output file          */

   if ( Verbose )
   {
      fprintf ( stderr, "\n%s wrote %d bytes to file %s\n",
                progname, i, OutFileName ) ;

        fprintf ( stderr, "AltAcc  CheckSum: %u = %02x %02x\n",
                   AltAccSum, InpBuffer [   i-4 ] & 0x00FF,
                                   InpBuffer [ i-3 ] & 0x00FF );

      fprintf ( stderr, "ProRead CheckSum: %u = %02x %02x\n",
                ChkSum, ChkSum & 0x00FF, ( ChkSum & 0xFF00 ) >> 8 );
   }

   if ( i != ALTACC_DUMP_LEN )
   {
      fprintf (stderr, "*** Warning ***  File Size error reading AltAcc !\n" ) ;

      if ( YesNo   (    "    Try Again ( y|n ) ? " ) != 'n' )
         goto ProReadLoop ;
      else
         exit ( 5 ) ;
   }
   if ( ChkSum != AltAccSum )
   {
      fprintf (stderr, "*** Warning ***  Check Sum error reading AltAcc !\n" ) ;

      if ( YesNo   (    "    Try Again ( y|n ) ? " ) != 'n' )
         goto ProReadLoop ;
      else
         exit ( 5 ) ;
   }
/*
   if ( Verbose )
   {
      sprintf ( InpBuffer, "%s/produce %s", proghome, OutFileName ) ;
      system ( InpBuffer ) ;
   }
*/
   exit ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
void  Usage ()
/* ------------------------------------------------------------------------ */
{
   fprintf ( stderr, "%s -- Reads flight data stored in AltAcc EEProm to a file\n\n",
             progname ) ;
   fprintf ( stderr, "Usage:  %s [-p COM#] [-f FILE] [-q] [-o FILE] -h  FILE\n",
             progname );
   fprintf ( stderr, "where:\n" );
   fprintf ( stderr, "        -p COM# -> substitute { COM1, COM2, COM3, COM4 }\n" );
   fprintf ( stderr, "        -n FILE -> Override default init file prodata.nit with FILE\n" );
   fprintf ( stderr, "        -o FILE -> FILE == local file name for the AltAcc data\n" );
   fprintf ( stderr, "        -q      -> be quiet about it\n" ) ;
   fprintf ( stderr, "        -h      -> help ( print this list )\n" );
   fprintf ( stderr, "           FILE -> FILE == local file name for the AltAcc data\n" );
   fprintf ( stderr, "                           ( same effect as the -o FILE option )\n" );

   exit ( 6 ) ;
}
/* ------------------------------------------------------------------------ */
void   CleanUp ()
/* ------------------------------------------------------------------------ */
{
   if ( ComOwnd )
      tcsetattr ( ComPort, TCSANOW, &otermios );

   /* atexit was exec'd after we took the [Ctrl][Break] IRQ ... so ...      */

   signal ( SIGINT, SIG_IGN );      /* and let dos do whatever it do    */
}
/* ------------------------------------------------------------------------ */
void  Breaker ( sig )
/* ------------------------------------------------------------------------ */
      int   sig ;
/* ------------------------------------------------------------------------ */
{
   /* A signal handler must take a single argument. The argument can be
    * tested within the handler and thus allows a single signal handler
    * to handle several different signals. In this case, the parameter
    * is included to keep the compiler from generating a warning but is
    * ignored because this signal handler only handles one interrupt:
    * SIGINT (Ctrl+C).
    */

   /* Disallow CTRL+C during handler. */

   signal ( SIGINT, SIG_IGN );

   Exit_YesNo ( "Break Detected - abort processing (y|n)? " );

   /* The CTRL+C interrupt must be reset to our handler since
    * by default it is reset to the system handler.
    */

   signal ( SIGINT, Breaker );
}
/* ------------------------------------------------------------------------ */
void   SafeOut ( str )         /* Outputs a string using system level calls. */
/* ------------------------------------------------------------------------ */
      char *str ;
/* ------------------------------------------------------------------------ */
{
   fputs ( str, stdout ) ;
}
/* ------------------------------------------------------------------------ */
int   SafeIn ()             /* Inputs a character using system level calls. */
/* ------------------------------------------------------------------------ */
{
   return ( fgetc ( stdin )) ;
}
/* ------------------------------------------------------------------------ */
char  YesNo ( Mess )
/* ------------------------------------------------------------------------ */
      char *  Mess   ;
/* ------------------------------------------------------------------------ */
{
   int c ;
   char str [] = " " ;

   SafeOut ( Mess );             /* ask dos to dump the message */

   c = SafeIn ();                /* ask dos for an input char   */
   str [0] = c;                  /* make a string out of it ... */
   SafeOut ( str );              /* and ask dos to dump it      */
   SafeOut ( "\r\n" );

   if ( isupper ( c ))
      c = tolower ( c ) ;

   return ( c ) ;
}
/* ------------------------------------------------------------------------ */
void   Exit_YesNo ( Mess )
/* ------------------------------------------------------------------------ */
      char * Mess   ;
/* ------------------------------------------------------------------------ */
{
   char c ;

   c = YesNo ( Mess ) ;

   if ( c == 'y' )
      exit ( 3 ) ;
}
/* ------------------------------------------------------------------------ */
void Sleep ( wait )
/* ------------------------------------------------------------------------ */
     double   wait ;
/* ------------------------------------------------------------------------ */
{
    time_t start, goal ;

    time ( & start ) ;

    do
    {
        time ( & goal ) ;

    } while (( difftime ( goal, start )) < wait ) ;
}
/* ------------------------------------------------------------------------ */
u16   Parse (line, maxarg, word)
/* ------------------------------------------------------------------------ */
      char * line;
      u16    maxarg;
      char * word [];
/* ------------------------------------------------------------------------ */
{
   u16      i = 0;
   char   * j = line + strlen (line);

   while (( line < j ) && ( i < maxarg ))
   {
      while (( line < j ) && ( isspace ( *line )))
         line++ ;

      if (( line >= j ) || ( *line == COMMENT_CHAR ))
         break;

      word [ i++ ] = line;

      while (( line < j ) && !( isspace ( *line )))
         line++ ;

      if ( line >= j )
         break;

      *line++ = '\0';
  }

  return ( i ) ;
}
/* ------------------------------------------------------------------------ */
char   * ToLower ( InStr )
/* ------------------------------------------------------------------------ */
   char * InStr ;
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;

   while ( i < strlen ( InStr ))
   {
      if ( isupper ( InStr [ i ] ))
         InStr [ i ]  = tolower ( InStr [ i ] ) ;

      i ++ ;
   }

   return ( InStr ) ;

}
/* ------------------------------------------------------------------------ */
char   * GetMyName ( Name,  ArgV )
/* ------------------------------------------------------------------------ */
   char * Name ;
   char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;

   /* Throw out the path */

   if (( Name = strrchr ( ArgV, DIR_SEP )) != NULL )
   {
      * Name = '\0' ;
        Name ++ ;
   }
   else
      Name = ArgV ;

   /* throw away the '.exe.' */

   if (( p = strrchr ( Name, '.' )) != NULL )
      * p = '\0' ;

   return ( Name ) ;

}
/* ------------------------------------------------------------------------ */
int  IsInList ( Char, List )
/* ------------------------------------------------------------------------ */
      byte   Char ;
      char * List ;
/* ------------------------------------------------------------------------ */
{
   int i = 0 ;

   for ( i = 0 ; i < strlen ( List ) ; i ++ )
      if ( Char == List [i] )
         return ( 1 ) ;

   return ( 0 ) ;
}
/* ------------------------------------------------------------------------ */
char  * GetMyHome ( Path,  ArgV )
/* ------------------------------------------------------------------------ */
      char * Path ;
      char * ArgV ;
/* ------------------------------------------------------------------------ */
{
   char * p ;
   int    i = 0 ;
   int    j = 0 ;

   /* First look for a Home for the System Environment */

   if (( p = getenv ( "PRODATA" )) != NULL )
      return ( p ) ;

   /* None found, try for a fqpn */

   Path = ArgV ;

   p = ArgV + strlen ( ArgV ) - 1;

   while ( p >= Path )
   {
      if ( IsInList ( * p, "/\\" ))
         * p = '\0' ;
      else
         break ;

      p -- ;
   }

   /* strip off /..'s  and count the occurances */

   while ( 1 )
   {
      i = strlen ( Path ) - 1 ;

      if (( i > 1 ) && ( Path [i] == '.' ) &&
                       ( Path [i-1] == '.' ) &&
                       ( Path [i-2] == DIR_SEP ))
      {
         Path [i-2] = '\0' ;
         j ++ ;
      }
      else
         break ;
   }

   /* now trim the corresponding dir's  */

   i = 0 ;

   while ( i < j )
   {
      if (( p = strrchr ( Path, DIR_SEP )) != NULL )
            * p = '\0' ;
      i ++ ;
   }

   /* a unix system puts the program name in the env sans path, so ... */

   if ( strcmp ( Path, progname ) != 0 )
      return ( Path ) ;
   else
      return ( "." ) ;

}
/* ------------------------------------------------------------------------ */
void  Initialize ( )
/* ------------------------------------------------------------------------ */
{
   u16   i = 0 ;
   u16   j = 0 ;

#ifndef INP_SIZE
   #define INP_SIZE    80
   #define MAXARG       3
#endif

   char     InpBuf [ INP_SIZE+1 ] ;
   char  *  ArgBuf [ MAXARG ] ;

   u16      NumArg = 0;

   if (( NitFileAddr = fopen ( NitFileName, "rb" )) == NULL )
   {
      fprintf ( stderr, "\n%s cannot initialization file %s for input\n",
                           progname, NitFileName ) ;
   }
   else
   {
      /* all proread cares about is the serial port id */
      /* intentionally ignore all other nit names      */
      while ( ! feof ( NitFileAddr ))
      {
         if ( fgets ( InpBuf, INP_SIZE, NitFileAddr ) != NULL )
         {
            if ( Parse ( InpBuf, MAXARG, ArgBuf ) > 1 )
            {
               ToLower ( ArgBuf [0] ) ;

               for ( i = 0 ; i < NUM_NIT_TAGS ; i ++ )
                  if ( strcmp ( NitTags [i].Tag, ArgBuf [0] ) == 0 )
                     break ;

               if ( i < NUM_NIT_TAGS )
               {
                  switch ( NitTags [i].Ptr )
                  {
                     case NT_PORT:

                        if ( ! ComOride )
                        {
                           strncpy ( ComFileName, 
                                     ArgBuf [1], 
                                     NAME_LEN_MAX );
                        }
                        break ;

                     default:
                        break ;
                  }
               }
            }
         }
      }
      fclose ( NitFileAddr ) ;
   }
}
